# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class Axis(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'bounds': 'list[str]',
        'label': 'str',
        'prefix': 'str',
        'suffix': 'str',
        'base': 'str',
        'scale': 'AxisScale'
    }

    attribute_map = {
        'bounds': 'bounds',
        'label': 'label',
        'prefix': 'prefix',
        'suffix': 'suffix',
        'base': 'base',
        'scale': 'scale'
    }

    def __init__(self, bounds=None, label=None, prefix=None, suffix=None, base=None, scale=None):  # noqa: E501,D401,D403
        """Axis - a model defined in OpenAPI."""  # noqa: E501
        self._bounds = None
        self._label = None
        self._prefix = None
        self._suffix = None
        self._base = None
        self._scale = None
        self.discriminator = None

        if bounds is not None:
            self.bounds = bounds
        if label is not None:
            self.label = label
        if prefix is not None:
            self.prefix = prefix
        if suffix is not None:
            self.suffix = suffix
        if base is not None:
            self.base = base
        if scale is not None:
            self.scale = scale

    @property
    def bounds(self):
        """Get the bounds of this Axis.

        The extents of the axis in the form [lower, upper]. Clients determine whether bounds are inclusive or exclusive of their limits.

        :return: The bounds of this Axis.
        :rtype: list[str]
        """  # noqa: E501
        return self._bounds

    @bounds.setter
    def bounds(self, bounds):
        """Set the bounds of this Axis.

        The extents of the axis in the form [lower, upper]. Clients determine whether bounds are inclusive or exclusive of their limits.

        :param bounds: The bounds of this Axis.
        :type: list[str]
        """  # noqa: E501
        self._bounds = bounds

    @property
    def label(self):
        """Get the label of this Axis.

        Description of the axis.

        :return: The label of this Axis.
        :rtype: str
        """  # noqa: E501
        return self._label

    @label.setter
    def label(self, label):
        """Set the label of this Axis.

        Description of the axis.

        :param label: The label of this Axis.
        :type: str
        """  # noqa: E501
        self._label = label

    @property
    def prefix(self):
        """Get the prefix of this Axis.

        Label prefix for formatting axis values.

        :return: The prefix of this Axis.
        :rtype: str
        """  # noqa: E501
        return self._prefix

    @prefix.setter
    def prefix(self, prefix):
        """Set the prefix of this Axis.

        Label prefix for formatting axis values.

        :param prefix: The prefix of this Axis.
        :type: str
        """  # noqa: E501
        self._prefix = prefix

    @property
    def suffix(self):
        """Get the suffix of this Axis.

        Label suffix for formatting axis values.

        :return: The suffix of this Axis.
        :rtype: str
        """  # noqa: E501
        return self._suffix

    @suffix.setter
    def suffix(self, suffix):
        """Set the suffix of this Axis.

        Label suffix for formatting axis values.

        :param suffix: The suffix of this Axis.
        :type: str
        """  # noqa: E501
        self._suffix = suffix

    @property
    def base(self):
        """Get the base of this Axis.

        Radix for formatting axis values.

        :return: The base of this Axis.
        :rtype: str
        """  # noqa: E501
        return self._base

    @base.setter
    def base(self, base):
        """Set the base of this Axis.

        Radix for formatting axis values.

        :param base: The base of this Axis.
        :type: str
        """  # noqa: E501
        self._base = base

    @property
    def scale(self):
        """Get the scale of this Axis.

        :return: The scale of this Axis.
        :rtype: AxisScale
        """  # noqa: E501
        return self._scale

    @scale.setter
    def scale(self, scale):
        """Set the scale of this Axis.

        :param scale: The scale of this Axis.
        :type: AxisScale
        """  # noqa: E501
        self._scale = scale

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, Axis):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other
