# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class DashboardQuery(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'text': 'str',
        'edit_mode': 'QueryEditMode',
        'name': 'str',
        'builder_config': 'BuilderConfig'
    }

    attribute_map = {
        'text': 'text',
        'edit_mode': 'editMode',
        'name': 'name',
        'builder_config': 'builderConfig'
    }

    def __init__(self, text=None, edit_mode=None, name=None, builder_config=None):  # noqa: E501,D401,D403
        """DashboardQuery - a model defined in OpenAPI."""  # noqa: E501
        self._text = None
        self._edit_mode = None
        self._name = None
        self._builder_config = None
        self.discriminator = None

        if text is not None:
            self.text = text
        if edit_mode is not None:
            self.edit_mode = edit_mode
        if name is not None:
            self.name = name
        if builder_config is not None:
            self.builder_config = builder_config

    @property
    def text(self):
        """Get the text of this DashboardQuery.

        The text of the Flux query.

        :return: The text of this DashboardQuery.
        :rtype: str
        """  # noqa: E501
        return self._text

    @text.setter
    def text(self, text):
        """Set the text of this DashboardQuery.

        The text of the Flux query.

        :param text: The text of this DashboardQuery.
        :type: str
        """  # noqa: E501
        self._text = text

    @property
    def edit_mode(self):
        """Get the edit_mode of this DashboardQuery.

        :return: The edit_mode of this DashboardQuery.
        :rtype: QueryEditMode
        """  # noqa: E501
        return self._edit_mode

    @edit_mode.setter
    def edit_mode(self, edit_mode):
        """Set the edit_mode of this DashboardQuery.

        :param edit_mode: The edit_mode of this DashboardQuery.
        :type: QueryEditMode
        """  # noqa: E501
        self._edit_mode = edit_mode

    @property
    def name(self):
        """Get the name of this DashboardQuery.

        :return: The name of this DashboardQuery.
        :rtype: str
        """  # noqa: E501
        return self._name

    @name.setter
    def name(self, name):
        """Set the name of this DashboardQuery.

        :param name: The name of this DashboardQuery.
        :type: str
        """  # noqa: E501
        self._name = name

    @property
    def builder_config(self):
        """Get the builder_config of this DashboardQuery.

        :return: The builder_config of this DashboardQuery.
        :rtype: BuilderConfig
        """  # noqa: E501
        return self._builder_config

    @builder_config.setter
    def builder_config(self, builder_config):
        """Set the builder_config of this DashboardQuery.

        :param builder_config: The builder_config of this DashboardQuery.
        :type: BuilderConfig
        """  # noqa: E501
        self._builder_config = builder_config

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, DashboardQuery):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other
